@@ -13,6 +13,27 @@ nobs.survreg <- function(object, ...)
 weights.survreg <- function(object, ...)
   model.weights(model.frame(object))
 
+## fix vcov() and bread() methods because some versions of survival
+## do not report row/column names anymore
+vcov.tobit <- function(object, ...) {
+  vc <- NextMethod()
+  if(is.null(colnames(vc))) {
+    nam <- names(object$coefficients)    
+    nam <- if(length(nam) == ncol(vc)) {
+      nam
+    } else if(length(nam) == ncol(vc) - 1L) {
+      c(nam, "Log(scale)")
+    } else {
+      c(nam, names(object$scale))
+    }
+    colnames(vc) <- rownames(vc) <- nam
+  }
+  return(vc)  
+}
+
+bread.tobit <- function(x, ...) {
+  length(x$linear.predictors) * vcov(x)
+}
 
 ## convenience tobit() interface to survreg()
 
@@ -145,9 +166,21 @@ summary.tobit <- function(object, correlation = FALSE, symbolic.cor = FALSE, vco
     else pprint <- paste(sd$name, "distribution")
 
   ## number of observations
-  nobs <- as.vector(table(factor(object$y[,2], levels = 2:0)))
-  nobs <- c(sum(nobs), nobs)
-  names(nobs) <- c("Total", "Left-censored", "Uncensored", "Right-censored")
+  ## (incorporating "bug fix" change for $y in survival 2.42-7)
+  surv_table <- function(y) {
+    if(!inherits(y, "Surv")) y <- y$y
+    type <- attr(y, "type")
+    if(is.null(type) || (type == "left" && any(y[, 2L] > 1))) type <- "old"
+    y <- switch(type,    
+      "left" = 2 - y[, 2L],
+      "interval" = y[, 3L],
+      y[, 2L]
+    )
+    table(factor(y, levels = c(2, 1, 0, 3),
+      labels = c("Left-censored", "Uncensored", "Right-censored", "Interval-censored")))
+  }
+  nobs <- surv_table(object$y)
+  nobs <- c("Total" = sum(nobs), nobs[1:3])
 
   rval <- object[match(c("call", "df", "loglik", "iter", "na.action", "idf", "scale"),
     names(object), nomatch = 0)]
@@ -267,15 +300,12 @@ lrtest.tobit <- function(object, ..., name = NULL)
 linearHypothesis.tobit <- function(model, hypothesis.matrix,
   rhs = NULL, vcov. = NULL, ...)
 {
-  if(compareVersion(packageDescription("car")$Version, "2.0-19") < 0L) {
-    stop("The linearHypothesis() method requires package car version 2.0-19 or later.")
-  }
   if(is.null(vcov.)) {
     vcov. <- vcov(model)
   } else {
     if(is.function(vcov.)) vcov. <- vcov.(model)
   }
-  vcov. <- vcov.[-nrow(vcov.), -ncol(vcov.)]
+  if("Log(scale)" %in% rownames(vcov.)) vcov. <- vcov.[-nrow(vcov.), -ncol(vcov.)]
   model$formula <- model$call$formula
   car::linearHypothesis.default(model,
     hypothesis.matrix = hypothesis.matrix, rhs = rhs, vcov. = vcov., ...)
